#include "driverEnc28.h"

//------------------------------------------------------------------------------
ENC28J60::ENC28J60( SPI *_spi, Pin *_res, Pin *_cs )
{
  spi = _spi;
  res = _res;
  cs  = _cs;
  idChip = 0;
  currentBunk = 0;
}
//------------------------------------------------------------------------------
void ENC28J60::enaChip()
{
  spi->sendByte( 0x00 );
  cs->Clear();
}
//------------------------------------------------------------------------------
void ENC28J60::disChip()
{
  spi->sendByte( 0x00 );
  cs->Set();
}
//------------------------------------------------------------------------------
void ENC28J60::setRST()
{
  res->Clear();
  HAL_Delay( 200 );
}
//------------------------------------------------------------------------------
void ENC28J60::clrRST()
{
  res->Set();
  HAL_Delay( 200 );
}
//------------------------------------------------------------------------------
void ENC28J60::init( uint8_t *macAddr )
{
  setRST();
  clrRST();
  
  resetSystem();
  HAL_Delay(100);
  
  writePhyReg( PHLCON, 0x476 );
  
  // configuration of RX buffer 
  writeControlReg16( ERXST, RX_BUF_START );
  writeControlReg16( ERXND, RX_BUF_END );
  writeControlReg16( ERXRDPT, RX_BUF_START); // is register being programming? 

  writeControlReg( MACON1, MACON1_MARXEN );
  writeControlReg( MACON1, MACON1_TXPAUS );
  writeControlReg( MACON1, MACON1_RXPAUS );
  writeControlReg( MACON3, MACON3_PADCFG0 );
  writeControlReg( MACON3, MACON3_TXCRCEN );
  writeControlReg( MACON3, MACON3_FRMLNEN );
  writeControlReg( MACON3, MACON3_FULDPX );
  
  writeControlReg16( MAMXFL, MAXFRAME );
  writeControlReg( MABBIPG, 0x15 );
  writeControlReg( MAIPG, 0x12 );
  writeControlReg( MAIPG + 1, 0x0C );
  
  writeControlReg( MAADR5, macAddr[0] );
  writeControlReg( MAADR4, macAddr[1] );
  writeControlReg( MAADR3, macAddr[2] );
  writeControlReg( MAADR2, macAddr[3] );
  writeControlReg( MAADR1, macAddr[4] );
  writeControlReg( MAADR0, macAddr[5] );
  
  writePhyReg( PHCON2, PHCON2_HDLDIS );
  writePhyReg( PHCON1, PHCON1_PDPXMD );
  
  bitFieldSet( ECON1, ECON1_RXEN );
  
  idChip = readControlReg( EREVID );
}
//------------------------------------------------------------------------------
void ENC28J60::sendPackage( uint16_t len )
{
  while ( readControlReg( ECON1 ) & ECON1_TXRTS )
  {
    if ( readControlReg( EIR ) & EIR_TXERIF )
    {
      bitFieldSet( ECON1, ECON1_TXRTS );
      bitFieldClear( ECON1, ECON1_TXRTS );
    }
  }
  
  writeControlReg16( EWRPT, TX_BUF_START );
  writeControlReg16( ETXST, TX_BUF_START );
  writeControlReg16( ETXND, TX_BUF_START + len );
  uint8_t controlByte = 0x00;
  writeBufferMem( &controlByte, 1 );
  writeBufferMem( txBuffer, len );
  
  bitFieldSet( ECON1, ECON1_TXRTS );
  
  uint16_t etxnd = readControlReg16( ETXND );
//  uint16_t erxst = readControlReg16( ERXST );
//  uint16_t erxnd = readControlReg16( ERXND );
//  uint16_t var   = readControlReg16( ERXND );
  writeControlReg16( ERDPT, etxnd + 1 );
  readBufferMem( txStatusVector, 7 );
}
//------------------------------------------------------------------------------
uint16_t ENC28J60::receivePackage( )
{
  static uint16_t currentRxPtr = RX_BUF_START;
  uint16_t len;
  
  if ( readControlReg( EPKTCNT ) > 0 ) 
  {
    writeControlReg( ERDPT, currentRxPtr );
    
    struct 
    {
      uint16_t nextPackage;
      uint16_t length;
      uint16_t status;
    } header;
    
    readBufferMem( (uint8_t*)&header, sizeof( header ) );
    currentRxPtr = header.nextPackage - 1;
    
    if ( header.status & 0x80 )
    {
      len = header.length - 4;
      
      if ( len > RX_SIZE ) len = RX_SIZE;
      readBufferMem( rxBuffer, len );
    }
    
    uint16_t nextPtr = header.nextPackage - 1;
    if ( nextPtr > RX_BUF_END ) nextPtr = RX_BUF_END;
    
    bitFieldSet( ECON2, ECON2_PKTDEC );
  }
  
  return len;
}
//------------------------------------------------------------------------------
void ENC28J60::setTxBuffer( const void *data, uint16_t len )
{
  const uint8_t *p = static_cast<const uint8_t*>( data );
  
  for ( uint8_t i = 0; i < len; i++ ) 
    txBuffer[i] = p[i];
}
//------------------------------------------------------------------------------
void ENC28J60::getRxBuffer( void *data, uint16_t len )
{
  uint8_t *p = static_cast<uint8_t*>( data );
  
  for ( uint8_t i = 0; i < len; i++ ) 
    p[i] = rxBuffer[i];
}
//------------------------------------------------------------------------------
//---------------------------------Private function-----------------------------
//------------------------------------------------------------------------------
void ENC28J60::checkBank( uint8_t address )
{
  if ( (address & BANK_MASK) != currentBunk )
  {
    write( ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_BSEL1 | ECON1_BSEL0 );
    currentBunk = address & BANK_MASK;
    write( ENC28J60_BIT_FIELD_SET, ECON1, currentBunk >> 5 );
  }
}
//------------------------------------------------------------------------------
void ENC28J60::write( uint8_t opCode, uint8_t address, uint8_t data )
{
//  enaChip();
  spi->sendByte( opCode | (address & ADDR_MASK) );
  spi->sendByte( data );
//  disChip();
}
//------------------------------------------------------------------------------
uint8_t ENC28J60::read( uint8_t opCode, uint8_t address )
{
  uint8_t data;
  
//  enaChip();
  spi->sendByte( opCode | (address & ADDR_MASK) );
  if ( address & SPRD_MASK ) spi->sendByte( 0x00 );
  data = spi->readByte();
//  disChip();
  
  return data;
}
//------------------------------------------------------------------------------
uint8_t ENC28J60::readControlReg( uint8_t address )
{
  uint8_t data;
  
  enaChip();
  checkBank( address );
  data = read( ENC28J60_READ_CTRL_REG, address );
  disChip();
  
  return data;
}
//------------------------------------------------------------------------------
uint16_t ENC28J60::readControlReg16( uint8_t address )
{
  uint16_t data;
  
  enaChip();
  checkBank( address );
  data = read( ENC28J60_READ_CTRL_REG, address ) | 
         (read( ENC28J60_READ_CTRL_REG, address + 1 ) << 8);
  disChip();
  
  return data;
}
//------------------------------------------------------------------------------
void ENC28J60::readBufferMem( uint8_t* data, uint16_t len )
{
  enaChip();
  spi->sendByte( ENC28J60_READ_BUF_MEM );  
  spi->readData( data, len );
  disChip();
}
//------------------------------------------------------------------------------
void ENC28J60::writeControlReg( uint8_t address, uint8_t data )
{
  enaChip();
  checkBank( address );
  write( ENC28J60_WRITE_CTRL_REG, address, data );
  disChip();
}
//------------------------------------------------------------------------------
void ENC28J60::writeControlReg16( uint8_t address, uint16_t data )
{
  enaChip();
  checkBank( address );
  write( ENC28J60_WRITE_CTRL_REG, address, data );
  write( ENC28J60_WRITE_CTRL_REG, address + 1, data >> 8);
  disChip();
}
//------------------------------------------------------------------------------
void ENC28J60::writeBufferMem( const void* data, uint16_t len )
{
  enaChip();
  spi->sendByte( ENC28J60_WRITE_BUF_MEM );
  spi->sendData( data, len );
  disChip();
}
//------------------------------------------------------------------------------
void ENC28J60::bitFieldSet( uint8_t address, uint8_t data )
{
  enaChip();
  checkBank( address );
  write( ENC28J60_BIT_FIELD_SET, address, data ); 
  disChip();
}
//------------------------------------------------------------------------------
void ENC28J60::bitFieldClear( uint8_t address, uint8_t data )
{
  enaChip();
  checkBank( address );
  write( ENC28J60_BIT_FIELD_CLR, address, data ); 
  disChip();
}
//------------------------------------------------------------------------------
void ENC28J60::resetSystem()
{
  enaChip();
  write( ENC28J60_SOFT_RESET, 0, ENC28J60_SOFT_RESET );
  disChip();
  currentBunk = 0;
  HAL_Delay(2);
}
//------------------------------------------------------------------------------
uint16_t ENC28J60::readPhyReg( uint8_t address )
{
  enaChip();
  writeControlReg( MIREGADR, address );
  writeControlReg( MICMD, MICMD_MIISCAN );
  while ( readControlReg(MISTAT) & MISTAT_BUSY );  
  writeControlReg( MICMD, 0x00 );
  disChip();
  return readControlReg( address ) | (readControlReg( address + 1 ) << 8);
}
//------------------------------------------------------------------------------
void ENC28J60::writePhyReg( uint8_t address, uint16_t data )
{
  enaChip();
  writeControlReg( MIREGADR, address );
  writeControlReg( MIWR, data );
  while ( readControlReg(MISTAT) & 0 ); 
  disChip();
}
//------------------------------------------------------------------------------








